<?php

namespace Drupal\tooltip_modal_popup\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\Markup;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\node\NodeInterface;
use Drupal\token\TokenInterface;
use Drupal\tooltip_modal_popup\Form\TooltipSettingsForm;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Class for modal window.
 */
class TooltipModalWindow extends ControllerBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(protected TokenInterface $tokenService, protected RequestStack $requestStack, protected RouteMatchInterface $routeMatch) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
          $container->get('token'),
          $container->get('request_stack'),
          $container->get('current_route_match'),
      );
  }

  /**
   * {@inheritdoc}
   */
  public function displayModal(?int $id = NULL): AjaxResponse {

    $config = $this->config(TooltipSettingsForm::SETTINGS);
    $height = 600;
    $width = 800;

    $content = Markup::create($this->t('Not Found.'));

    $config_data = !empty($config->get('serialize_tooltip_data')) ? unserialize($config->get('serialize_tooltip_data'), ['allowed_classes' => FALSE]) : [];

    if (isset($config_data[$id]['tooltip_content']['value']) && !empty($config_data[$id]['tooltip_content']['value'])) {
      $user = $this->entityTypeManager()->getStorage('user')->load($this->currentUser()->id());
      $node = $this->routeMatch->getParameter('node');
      if (!($node instanceof NodeInterface)) {
        $request = $this->requestStack->getCurrentRequest();
        $referer = $request->headers->get('referer');
        if ($referer) {
          $url_parts = parse_url($referer);
          if (isset($url_parts['path'])) {
            $path = $url_parts['path'];
            if (preg_match('/\/node\/(\d+)/', $path, $matches)) {
              $nid = $matches[1];
              $node = $this->entityTypeManager()->getStorage('node')->load($nid);
            }
          }
        }
      }

      $token_data = [
        'node' => $node,
        'user' => $user,
      ];
      $token_options = ['clear' => TRUE];

      $content = $this->tokenService->replace($config_data[$id]['tooltip_content']['value'], $token_data, $token_options);
    }

    $response = new AjaxResponse();
    $response->addCommand(new OpenModalDialogCommand("", $content, ['height' => $height, 'width' => $width]));
    return $response;
  }

}
