<?php

namespace Drupal\tooltip_modal_popup\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * Define class TooltipSettingsForm.
 */
class TooltipSettingsForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
  */
  const SETTINGS = 'tooltip_modal_popup.settings';

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'tooltip_modal_popup_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = !empty($this->config(static::SETTINGS)->get('serialize_tooltip_data')) ? unserialize($this->config(static::SETTINGS)->get('serialize_tooltip_data'), ['allowed_classes' => FALSE]) : [];

    $last_element = end($config);
    if (!empty($last_element['js_selector']) || count($config) == 0) {
      $config[] = [
        'js_selector' => '',
        'tooltip' => '',
        'tooltip_label' => '',
        'tooltip_content' => '',
      ];
    }

    $header = [
      'id' => $this->t('ID'),
      'js_selector' => $this->t('Selector'),
      'tooltip' => $this->t('Tooltip'),
      'tooltip_label' => $this->t('Tooltip Label'),
      'tooltip_content' => $this->t('Tooltip Content'),
    ];

    $form['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => [],
    ];

    foreach ($config as $key => $conf) {
      $js_selector = [
        '#type' => 'textfield',
        '#required' => FALSE,
        '#default_value' => $conf['js_selector'] ?? '',
      ];

      $tooltip = [
        '#type' => 'textfield',
        '#required' => FALSE,
        '#default_value' => $conf['tooltip'] ?? '',
      ];

      $tooltip_label = [
        '#type' => 'textfield',
        '#required' => FALSE,
        '#default_value' => $conf['tooltip_label'] ?? '',
      ];

      $tooltip_content = [
        '#type' => 'text_format',
        '#required' => FALSE,
        '#default_value' => $conf['tooltip_content']['value'] ?? '',
        '#format' => $conf['tooltip_content']['format'] ?? 'cp_full_html',
      ];

      $form['table'][] = [
        'id' => ['#markup' => $key + 1],
        'js_selector' => $js_selector,
        'tooltip' => $tooltip,
        'tooltip_label' => $tooltip_label,
        'tooltip_content' => $tooltip_content,
      ];
    }

    $form['token_browse'] = [
      '#theme' => 'token_tree_link',
      '#token_types' => ['node', 'user'],
      '#show_restricted' => TRUE,
      '#weight' => 90,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $arr_data = [];

    foreach ($form_state->getValue('table') as $data) {
      if (array_key_exists($data['js_selector'], $arr_data)) {
        $form_state->setErrorByName('js_selector', $this->t('Selector must be unique.'));
      }
      else {
        $arr_data[$data['js_selector']] = $data['js_selector'];
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $arr_data = $form_state->getValue('table');

    foreach ($arr_data as $key => $data) {
      if (empty($data['js_selector'])) {
        unset($arr_data[$key]);
      }
    }

    $config = $this->configFactory->getEditable(static::SETTINGS);
    $config->set('serialize_tooltip_data', serialize($arr_data));
    $config->save();

    parent::submitForm($form, $form_state);
  }

}
